diff --git a/files/etc/config/batman-adv b/files/etc/config/batman-adv
index fb7b384..f775d80 100644
--- a/files/etc/config/batman-adv
+++ b/files/etc/config/batman-adv
@@ -1,6 +1,5 @@
 
 config 'mesh' 'bat0'
-	option 'interfaces' 'mesh'
 	option 'aggregated_ogms'
 	option 'ap_isolation'
 	option 'bonding'
diff --git a/files/etc/hotplug.d/iface/99-batman-adv b/files/etc/hotplug.d/iface/99-batman-adv
deleted file mode 100644
index 9206c42..0000000
--- a/files/etc/hotplug.d/iface/99-batman-adv
+++ /dev/null
@@ -1,12 +0,0 @@
-#!/bin/sh
-
-. /lib/batman-adv/config.sh
-
-bat_load_module
-config_load batman-adv
-
-case "$ACTION" in
-        ifup)
-                [ -d /sys/class/net/$DEVICE/batman_adv/ ] && config_foreach bat_add_interface mesh "$DEVICE"
-                ;;
-esac
diff --git a/files/etc/hotplug.d/net/99-batman-adv b/files/etc/hotplug.d/net/99-batman-adv
index 47132c1..f0c391f 100644
--- a/files/etc/hotplug.d/net/99-batman-adv
+++ b/files/etc/hotplug.d/net/99-batman-adv
@@ -9,7 +9,4 @@ case "$ACTION" in
 	add)
 		[ -d /sys/class/net/$INTERFACE/mesh/ ] && bat_config "$INTERFACE"
 		;;
-	remove)
-		[ -d /sys/class/net/$INTERFACE/batman_adv/ ] && config_foreach bat_del_interface mesh "$INTERFACE"
-		;;
 esac
diff --git a/files/lib/batman-adv/config.sh b/files/lib/batman-adv/config.sh
index 313dd3a..d0810b8 100644
--- a/files/lib/batman-adv/config.sh
+++ b/files/lib/batman-adv/config.sh
@@ -41,37 +41,3 @@ bat_config()
 	[ -n "$ap_isolation" ] && echo $ap_isolation > /sys/class/net/$mesh/mesh/ap_isolation
 	[ -n "$bridge_loop_avoidance" ] && echo $bridge_loop_avoidance > /sys/class/net/$mesh/mesh/bridge_loop_avoidance
 }
-
-bat_add_interface()
-{
-	local mesh="$1"
-	local interface="$2"
-	local interfaces
-
-	config_get interfaces $mesh interfaces
-	for iface in $interfaces; do
-		[ -f "/sys/class/net/$iface/batman_adv/mesh_iface" ] || {
-			iface=$(uci -q -P/var/state get network.$iface.ifname)
-			[ -f "/sys/class/net/$iface/batman_adv/mesh_iface" ] || continue
-		}
-	
-		[ "$iface" = "$interface" ] && echo $mesh > /sys/class/net/$iface/batman_adv/mesh_iface
-	done
-}
-
-bat_del_interface()
-{
-	local mesh="$1"
-	local interface="$2"
-	local interfaces
-
-	config_get interfaces $mesh interfaces
-	for iface in $interfaces; do
-		[ -f "/sys/class/net/$iface/batman_adv/mesh_iface" ] || {
-			iface=$(uci -q -P/var/state get network.$iface.ifname)
-			[ -f "/sys/class/net/$iface/batman_adv/mesh_iface" ] || continue
-		}
-
-		[ "$iface" = "$interface" ] && echo none > /sys/class/net/$iface/batman_adv/mesh_iface
-	done
-}
diff --git a/files/lib/netifd/proto/batadv.sh b/files/lib/netifd/proto/batadv.sh
new file mode 100755
index 0000000..632a209
--- /dev/null
+++ b/files/lib/netifd/proto/batadv.sh
@@ -0,0 +1,30 @@
+#!/bin/sh
+
+. /lib/functions.sh
+. ../netifd-proto.sh
+init_proto "$@"
+
+proto_batadv_init_config() {
+	proto_config_add_string "mesh"
+}
+
+proto_batadv_setup() {
+	local config="$1"
+	local iface="$2"
+
+	local mesh
+	json_get_vars mesh
+
+	echo "$mesh" > "/sys/class/net/$iface/batman_adv/mesh_iface"
+	proto_init_update "$iface" 1
+	proto_send_update "$config"
+}
+
+proto_batadv_teardown() {
+	local config="$1"
+	local iface="$2"
+
+	echo "none" > "/sys/class/net/$iface/batman_adv/mesh_iface" || true
+}
+
+add_protocol batadv
